 
// Copyright � 2004 Apple Computer, Inc., All Rights Reserved

#ifndef __ComplexPlayThru_H__
#define __ComplexPlayThru_H__

#define checkErr( err) \
if(err) {\
	OSStatus error = static_cast<OSStatus>(err);\
		fprintf(stderr, "ComplexPlayThru Error: %ld ->  %s:  %d\n",  error,\
			   __FILE__, \
			   __LINE__\
			   );\
				   fflush(stdout);\
}         

#include <CoreAudio/CoreAudio.h>
#include <AudioToolbox/AudioToolbox.h>
#include <AudioUnit/AudioUnit.h>
#include "AudioRingBuffer2.h"
#include "AudioDevice.h"

class ComplexPlayThru 
{
public:
	ComplexPlayThru(AudioDeviceID input, AudioDeviceID output);
	~ComplexPlayThru();
	OSStatus Init(AudioDeviceID input, AudioDeviceID output);
	OSStatus Start();
	OSStatus Stop();
	Boolean IsRunning();
	OSStatus SetInputDeviceAsCurrent(AudioDeviceID in);
	OSStatus SetOutputDeviceAsCurrent(AudioDeviceID out);
private:
	OSStatus SetupGraph(AudioDeviceID out);
	OSStatus MakeGraph();
	OSStatus SetupAUHAL(AudioDeviceID in);
	OSStatus EnableIO();
	OSStatus CallbackSetup();
	OSStatus SetupBuffers();
	OSStatus MapChannels();
	
	void ComputeThruOffset();
	
	static OSStatus InputProc(void *inRefCon,
							  AudioUnitRenderActionFlags *ioActionFlags,
							  const AudioTimeStamp *inTimeStamp,
							  UInt32				inBusNumber,
							  UInt32				inNumberFrames,
							  AudioBufferList *		ioData);
	
	static OSStatus OutputProc(void *inRefCon,
							   AudioUnitRenderActionFlags *ioActionFlags,
							   const AudioTimeStamp *inTimeStamp,
							   UInt32				inBusNumber,
							   UInt32				inNumberFrames,
							   AudioBufferList *	ioData);
							   
	static OSStatus DeviceSampleRateListener(AudioDeviceID  inDevice,
								UInt32					inChannel,
								Boolean					isInput,
								AudioDevicePropertyID	inPropertyID,
								void*					inClientData);
	
	AudioUnit			mInputUnit;
	AudioBufferList		*mInputBuffer;
	AudioDevice			mInputDevice, mOutputDevice;
	AudioRingBuffer		*mBuffer;
	
	//AudioUnits and Graph
	AUGraph				mGraph;
	AUNode				mVarispeedNode;
	AudioUnit			mVarispeedUnit;
	AUNode				mOutputNode;
	AudioUnit			mOutputUnit;
	
	//Buffer sample info
	Float64 mFirstInputTime;
	Float64 mFirstOutputTime;
	Float64 mInToOutSampleOffset;
};


#endif //__ComplexPlayThru_H__
